@@ -18,7 +18,7 @@
 #ifndef IPKG_CMD_H
 #define IPKG_CMD_H
 
-typedef int (*ipkg_cmd_fun_t)(ipkg_conf_t *conf, int argc, char *argv[]);
+typedef int (*ipkg_cmd_fun_t)(ipkg_conf_t *conf, int argc, const char **argv);
 
 struct ipkg_cmd
 {
@@ -29,11 +29,18 @@ struct ipkg_cmd
 typedef struct ipkg_cmd ipkg_cmd_t;
 
 ipkg_cmd_t *ipkg_cmd_find(const char *name);
-int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, char *argv[]);
+#ifdef IPKG_LIB
+int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, 
+                  const char **argv, void *userdata);
+#else
+int ipkg_cmd_exec(ipkg_cmd_t *cmd, ipkg_conf_t *conf, int argc, const char **argv);
+#endif
 int ipkg_multiple_files_scan (ipkg_conf_t *conf, int argc, char *argv[]);
 /* install any packges with state_want == SW_INSTALL */
 int ipkg_install_wanted_packages(ipkg_conf_t *conf);
 /* ensure that all dependences are satisfied */
 int ipkg_configure_packages(ipkg_conf_t *conf, regex_t *re);
 
+int pkg_mark_provides(pkg_t *pkg);
+
 #endif
