@@ -17,6 +17,8 @@
 
 #include "ipkg.h"
 #include <ctype.h>
+#include <string.h>
+#include <errno.h>
 
 #include "pkg.h"
 
@@ -69,6 +71,7 @@ static const enum_map_t pkg_state_status_map[] = {
 
 static int verrevcmp(const char *val, const char *ref);
 
+
 pkg_t *pkg_new(void)
 {
      pkg_t *pkg;
@@ -369,7 +372,8 @@ abstract_pkg_t *abstract_pkg_new(void)
 	  return NULL;
      }
 
-     abstract_pkg_init(ab_pkg);
+     if ( abstract_pkg_init(ab_pkg) < 0 ) 
+        return NULL;
 
      return ab_pkg;
 }
@@ -379,39 +383,580 @@ int abstract_pkg_init(abstract_pkg_t *ab_pkg)
      memset(ab_pkg, 0, sizeof(abstract_pkg_t));
 
      ab_pkg->provided_by = abstract_pkg_vec_alloc();
+     if (ab_pkg->provided_by==NULL){
+        return -1;
+     }
      ab_pkg->dependencies_checked = 0;
      ab_pkg->state_status = SS_NOT_INSTALLED;
 
      return 0;
 }
-    
-void pkg_print_info(pkg_t *pkg, FILE *file)
-{
-     if (pkg == NULL) {
-	  return;
+
+void set_flags_from_control(ipkg_conf_t *conf, pkg_t *pkg){
+     char * temp_str;
+     char **raw =NULL;
+     char **raw_start=NULL; 
+
+     temp_str = (char *) malloc (strlen(pkg->dest->info_dir)+strlen(pkg->name)+12);
+     if (temp_str == NULL ){
+        ipkg_message(conf, IPKG_INFO, "Out of memory in  %s\n", __FUNCTION__);
+        return;
+     }
+     sprintf( temp_str,"%s/%s.control",pkg->dest->info_dir,pkg->name);
+   
+     raw = raw_start = read_raw_pkgs_from_file(temp_str);
+     if (raw == NULL ){
+        ipkg_message(conf, IPKG_ERROR, "Unable to open the control file in  %s\n", __FUNCTION__);
+        return;
      }
 
-     pkg_print_field(pkg, file, "Package");
-     pkg_print_field(pkg, file, "Version");
-     pkg_print_field(pkg, file, "Depends");
-     pkg_print_field(pkg, file, "Recommends");
-     pkg_print_field(pkg, file, "Suggests");
-     pkg_print_field(pkg, file, "Provides");
-     pkg_print_field(pkg, file, "Replaces");
-     pkg_print_field(pkg, file, "Conflicts");
-     pkg_print_field(pkg, file, "Status");
-     pkg_print_field(pkg, file, "Section");
-     pkg_print_field(pkg, file, "Essential"); /* @@@@ should be removed in future release. */
-     pkg_print_field(pkg, file, "Architecture");
-     pkg_print_field(pkg, file, "Maintainer");
-     pkg_print_field(pkg, file, "MD5sum");
-     pkg_print_field(pkg, file, "Size");
-     pkg_print_field(pkg, file, "Filename");
-     pkg_print_field(pkg, file, "Conffiles");
-     pkg_print_field(pkg, file, "Source");
-     pkg_print_field(pkg, file, "Description");
-     pkg_print_field(pkg, file, "Installed-Time");
-     fputs("\n", file);
+     while(*raw){
+        if (!pkg_valorize_other_field(pkg, &raw ) == 0) {
+            ipkg_message(conf, IPKG_DEBUG, "unable to read control file for %s. May be empty\n", pkg->name);
+        }
+     }
+     raw = raw_start;
+     while (*raw) {
+        if (raw!=NULL)
+          free(*raw++);
+     }
+
+     free(raw_start); 
+     free(temp_str);
+
+     return ;
+
+}
+
+char * pkg_formatted_info(pkg_t *pkg )
+{
+     char *line;
+     char * buff;
+
+     buff = malloc(8192);
+     if (buff == NULL) {
+	  fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	  return NULL;
+     }
+
+     buff[0] = '\0';
+
+     line = pkg_formatted_field(pkg, "Package");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Version");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Depends");
+     strncat(buff ,line, strlen(line));
+     free(line);
+     
+     line = pkg_formatted_field(pkg, "Recommends");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Suggests");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Provides");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Replaces");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Conflicts");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Status");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Section");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Essential"); /* @@@@ should be removed in future release. *//* I do not agree with this Pigi*/
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Architecture");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Maintainer");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "MD5sum");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Size");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Filename");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Conffiles");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Source");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Description");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     line = pkg_formatted_field(pkg, "Installed-Time");
+     strncat(buff ,line, strlen(line));
+     free(line);
+
+     return buff;
+}
+
+char * pkg_formatted_field(pkg_t *pkg, const char *field )
+{
+     static size_t LINE_LEN = 128;
+     char * temp = (char *)malloc(1);
+     int len = 0;
+
+/*
+  Pigi: After some discussion with Florian we decided to modify the full procedure in 
+        dynamic memory allocation. This should avoid any other segv in this area ( except for bugs )
+*/
+
+     if (strlen(field) < PKG_MINIMUM_FIELD_NAME_LEN) {
+	  goto UNKNOWN_FMT_FIELD;
+     }
+
+     temp[0]='\0'; 
+
+     switch (field[0])
+     {
+     case 'a':
+     case 'A':
+	  if (strcasecmp(field, "Architecture") == 0) {
+	       /* Architecture */
+	       if (pkg->architecture) {
+                   temp = (char *)realloc(temp,strlen(pkg->architecture)+17);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->architecture)+17), "Architecture: %s\n", pkg->architecture);
+	       }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+	  break;
+     case 'c':
+     case 'C':
+	  if (strcasecmp(field, "Conffiles") == 0) {
+	       /* Conffiles */
+	       conffile_list_elt_t *iter;
+               char confstr[LINE_LEN];
+
+	       if (pkg->conffiles.head == NULL) {
+		    return temp;
+	       }
+
+               len = 14 ;
+	       for (iter = pkg->conffiles.head; iter; iter = iter->next) {
+		    if (iter->data->name && iter->data->value) {
+                       len = len + (strlen(iter->data->name)+strlen(iter->data->value)+5);
+		    }
+	       }
+               temp = (char *)realloc(temp,len);
+               if ( temp == NULL ){
+	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	          return NULL;
+               }
+               temp[0]='\0';
+               strncpy(temp, "Conffiles:\n", 12);
+	       for (iter = pkg->conffiles.head; iter; iter = iter->next) {
+		    if (iter->data->name && iter->data->value) {
+                         snprintf(confstr, LINE_LEN, "%s %s\n", iter->data->name, iter->data->value);
+                         strncat(temp, confstr, strlen(confstr));           
+		    }
+	       }
+	  } else if (strcasecmp(field, "Conflicts") == 0) {
+	       int i;
+
+	       if (pkg->conflicts_count) {
+                    char conflictstr[LINE_LEN];
+                    len = 14 ;
+		    for(i = 0; i < pkg->conflicts_count; i++) {
+                        len = len + (strlen(pkg->conflicts_str[i])+5);
+                    }
+                    temp = (char *)realloc(temp,len);
+                    if ( temp == NULL ){
+	               fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	               return NULL;
+                    }
+                    temp[0]='\0';
+                    strncpy(temp, "Conflicts:", 11);
+		    for(i = 0; i < pkg->conflicts_count; i++) {
+                        snprintf(conflictstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->conflicts_str[i]);
+                        strncat(temp, conflictstr, strlen(conflictstr));           
+                    }
+                    strncat(temp, "\n", strlen("\n")); 
+	       }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+	  break;
+     case 'd':
+     case 'D':
+	  if (strcasecmp(field, "Depends") == 0) {
+	       /* Depends */
+	       int i;
+
+	       if (pkg->depends_count) {
+                    char depstr[LINE_LEN];
+                    len = 14 ;
+		    for(i = 0; i < pkg->depends_count; i++) {
+                        len = len + (strlen(pkg->depends_str[i])+4);
+                    }
+                    temp = (char *)realloc(temp,len);
+                    if ( temp == NULL ){
+	               fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	               return NULL;
+                    }
+                    temp[0]='\0';
+                    strncpy(temp, "Depends:", 10);
+		    for(i = 0; i < pkg->depends_count; i++) {
+                        snprintf(depstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->depends_str[i]);
+                        strncat(temp, depstr, strlen(depstr));           
+                    }
+                    strncat(temp, "\n", strlen("\n")); 
+	       }
+	  } else if (strcasecmp(field, "Description") == 0) {
+	       /* Description */
+	       if (pkg->description) {
+                   temp = (char *)realloc(temp,strlen(pkg->description)+16);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->description)+16), "Description: %s\n", pkg->description);
+	       }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+      break;
+     case 'e':
+     case 'E': {
+	  /* Essential */
+	  if (pkg->essential) {
+              temp = (char *)realloc(temp,16);
+              if ( temp == NULL ){
+	         fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	         return NULL;
+              }
+              temp[0]='\0';
+              snprintf(temp, (16), "Essential: yes\n");
+	  }
+     }
+	  break;
+     case 'f':
+     case 'F': {
+	  /* Filename */
+	  if (pkg->filename) {
+              temp = (char *)realloc(temp,strlen(pkg->filename)+12);
+              if ( temp == NULL ){
+	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	          return NULL;
+              }
+              temp[0]='\0';
+              snprintf(temp, (strlen(pkg->filename)+12), "Filename: %s\n", pkg->filename);
+	  }
+     }
+	  break;
+     case 'i':
+     case 'I': {
+	  if (strcasecmp(field, "Installed-Size") == 0) {
+	       /* Installed-Size */
+               temp = (char *)realloc(temp,strlen(pkg->installed_size)+17);
+               if ( temp == NULL ){
+	           fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	           return NULL;
+               }
+               temp[0]='\0';
+               snprintf(temp, (strlen(pkg->installed_size)+17), "Installed-Size: %s\n", pkg->installed_size);
+	  } else if (strcasecmp(field, "Installed-Time") == 0 && pkg->installed_time) {
+               temp = (char *)realloc(temp,29);
+               if ( temp == NULL ){
+	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	          return NULL;
+               }
+               temp[0]='\0';
+               snprintf(temp, 29, "Installed-Time: %ul\n", pkg->installed_time);
+	  }
+     }
+	  break;
+     case 'm':
+     case 'M': {
+	  /* Maintainer | MD5sum */
+	  if (strcasecmp(field, "Maintainer") == 0) {
+	       /* Maintainer */
+	       if (pkg->maintainer) {
+                   temp = (char *)realloc(temp,strlen(pkg->maintainer)+14);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->maintainer)+14), "maintainer: %s\n", pkg->maintainer);
+	       }
+	  } else if (strcasecmp(field, "MD5sum") == 0) {
+	       /* MD5sum */
+	       if (pkg->md5sum) {
+                   temp = (char *)realloc(temp,strlen(pkg->md5sum)+11);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->md5sum)+11), "MD5Sum: %s\n", pkg->md5sum);
+	       }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+     }
+	  break;
+     case 'p':
+     case 'P': {
+	  if (strcasecmp(field, "Package") == 0) {
+	       /* Package */
+               temp = (char *)realloc(temp,strlen(pkg->name)+11);
+               if ( temp == NULL ){
+	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	          return NULL;
+               }
+               temp[0]='\0';
+               snprintf(temp, (strlen(pkg->name)+11), "Package: %s\n", pkg->name);
+	  } else if (strcasecmp(field, "Priority") == 0) {
+	       /* Priority */
+               temp = (char *)realloc(temp,strlen(pkg->priority)+12);
+               if ( temp == NULL ){
+	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	          return NULL;
+               }
+               temp[0]='\0';
+               snprintf(temp, (strlen(pkg->priority)+12), "Priority: %s\n", pkg->priority);
+	  } else if (strcasecmp(field, "Provides") == 0) {
+	       /* Provides */
+	       int i;
+
+	       if (pkg->provides_count) {
+                    char provstr[LINE_LEN];
+                    len = 15;
+		    for(i = 0; i < pkg->provides_count; i++) {
+                        len = len + (strlen(pkg->provides_str[i])+5);
+                    }
+                    temp = (char *)realloc(temp,len);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                    temp[0]='\0';
+                    strncpy(temp, "Provides:", 12);
+		    for(i = 0; i < pkg->provides_count; i++) {
+                        snprintf(provstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->provides_str[i]);
+                        strncat(temp, provstr, strlen(provstr));           
+                    }
+                    strncat(temp, "\n", strlen("\n")); 
+               }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+     }
+	  break;
+     case 'r':
+     case 'R': {
+	  int i;
+	  /* Replaces | Recommends*/
+	  if (strcasecmp (field, "Replaces") == 0) {
+	       if (pkg->replaces_count) {
+                    char replstr[LINE_LEN];
+                    len = 14;
+		    for (i = 0; i < pkg->replaces_count; i++) {
+                        len = len + (strlen(pkg->replaces_str[i])+5);
+                    }
+                    temp = (char *)realloc(temp,len);
+                    if ( temp == NULL ){
+	               fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	               return NULL;
+                    }
+                    temp[0]='\0';
+                    strncpy(temp, "Replaces:", 12);
+		    for (i = 0; i < pkg->replaces_count; i++) {
+                        snprintf(replstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->replaces_str[i]);
+                        strncat(temp, replstr, strlen(replstr));           
+                    }
+                    strncat(temp, "\n", strlen("\n")); 
+	       }
+	  } else if (strcasecmp (field, "Recommends") == 0) {
+	       if (pkg->recommends_count) {
+                    char recstr[LINE_LEN];
+                    len = 15;
+		    for(i = 0; i < pkg->recommends_count; i++) {
+                         len = len + (strlen( pkg->recommends_str[i])+5);
+                    }
+                    temp = (char *)realloc(temp,len);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                    temp[0]='\0';
+                    strncpy(temp, "Recommends:", 13);
+		    for(i = 0; i < pkg->recommends_count; i++) {
+                        snprintf(recstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->recommends_str[i]);
+                        strncat(temp, recstr, strlen(recstr));           
+                    }
+                    strncat(temp, "\n", strlen("\n")); 
+	       }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+     }
+	  break;
+     case 's':
+     case 'S': {
+	  /* Section | Size | Source | Status | Suggests */
+	  if (strcasecmp(field, "Section") == 0) {
+	       /* Section */
+	       if (pkg->section) {
+                   temp = (char *)realloc(temp,strlen(pkg->section)+11);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->section)+11), "Section: %s\n", pkg->section);
+	       }
+	  } else if (strcasecmp(field, "Size") == 0) {
+	       /* Size */
+	       if (pkg->size) {
+                   temp = (char *)realloc(temp,strlen(pkg->size)+8);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->size)+8), "Size: %s\n", pkg->size);
+	       }
+	  } else if (strcasecmp(field, "Source") == 0) {
+	       /* Source */
+	       if (pkg->source) {
+                   temp = (char *)realloc(temp,strlen(pkg->source)+10);
+                   if ( temp == NULL ){
+	              fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	              return NULL;
+                   }
+                   temp[0]='\0';
+                   snprintf(temp, (strlen(pkg->source)+10), "Source: %s\n", pkg->source);
+               }
+	  } else if (strcasecmp(field, "Status") == 0) {
+	       /* Status */
+               temp = (char *)realloc(temp,strlen(pkg_state_want_to_str(pkg->state_want))+
+                               strlen(pkg_state_flag_to_str(pkg->state_flag))+
+                               strlen(pkg_state_status_to_str(pkg->state_status))+12);
+               if ( temp == NULL ){
+	          fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	          return NULL;
+               }
+               temp[0]='\0';
+               snprintf(temp, (strlen(pkg_state_want_to_str(pkg->state_want))+
+                               strlen(pkg_state_flag_to_str(pkg->state_flag))+
+                               strlen(pkg_state_status_to_str(pkg->state_status))+12), "Status: %s %s %s\n", 
+                                                             pkg_state_want_to_str(pkg->state_want),
+                                                             pkg_state_flag_to_str(pkg->state_flag),
+                                                             pkg_state_status_to_str(pkg->state_status));
+	  } else if (strcasecmp(field, "Suggests") == 0) {
+	       if (pkg->suggests_count) {
+		    int i;
+                    char sugstr[LINE_LEN];
+                    len = 13;
+		    for(i = 0; i < pkg->suggests_count; i++) {
+                        len = len + (strlen(pkg->suggests_str[i])+5);
+                    }
+                    temp = (char *)realloc(temp,len);
+                    if ( temp == NULL ){
+	               fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	               return NULL;
+                    }
+                    temp[0]='\0';
+                    strncpy(temp, "Suggests:", 10);
+		    for(i = 0; i < pkg->suggests_count; i++) {
+                        snprintf(sugstr, LINE_LEN, "%s %s", i == 0 ? "" : ",", pkg->suggests_str[i]);
+                        strncat(temp, sugstr, strlen(sugstr));           
+                    }
+                    strncat(temp, "\n", strlen("\n")); 
+	       }
+	  } else {
+	       goto UNKNOWN_FMT_FIELD;
+	  }
+     }
+	  break;
+     case 'v':
+     case 'V': {
+	  /* Version */
+	  char *version = pkg_version_str_alloc(pkg);
+          temp = (char *)realloc(temp,strlen(version)+14);
+          if ( temp == NULL ){
+	      fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+	      return NULL;
+          }
+          temp[0]='\0';
+          snprintf(temp, (strlen(version)+12), "Version: %s\n", version);
+	  free(version);
+     }
+	  break;
+     default:
+	  goto UNKNOWN_FMT_FIELD;
+     }
+
+     if ( strlen(temp)<2 ) {
+          temp[0]='\0';
+     }
+     return temp;
+
+ UNKNOWN_FMT_FIELD:
+     fprintf(stderr, "%s: ERROR: Unknown field name: %s\n", __FUNCTION__, field);
+     if ( strlen(temp)<2 ) {
+          temp[0]='\0';
+     }
+
+     return temp;
+}
+
+void pkg_print_info(pkg_t *pkg, FILE *file)
+{
+     char * buff;
+     if (pkg == NULL) {
+	return;
+     }
+
+     buff = pkg_formatted_info(pkg);
+     if ( buff == NULL ) 
+         return;
+     if (strlen(buff)>2){
+         fwrite(buff, 1, strlen(buff), file);
+     } 
+     free(buff);
 }
 
 void pkg_print_status(pkg_t * pkg, FILE * file)
@@ -431,7 +976,7 @@ void pkg_print_status(pkg_t * pkg, FILE * file)
 	from one handheld to another). So, maybe we actually want a few
 	more fields here, (depends, suggests, etc.), so that that would
 	be guaranteed to work even in the absence of more information
-	from the available file. 
+	from the available file.
 
 	28-MAR-03: kergoth and I discussed this yesterday.  We think
 	the essential info needs to be here for all installed packages
@@ -456,232 +1001,20 @@ void pkg_print_status(pkg_t * pkg, FILE * file)
      fputs("\n", file);
 }
 
-/* This function is extremely ugly and difficult to maintain.  It is
-   fast though, but is it really worth it? (Maybe, since it is here
-   and it works).  */
 void pkg_print_field(pkg_t *pkg, FILE *file, const char *field)
 {
+     char *buff;
      if (strlen(field) < PKG_MINIMUM_FIELD_NAME_LEN) {
-	  goto UNKNOWN_FIELD;
-     }
-
-     switch (field[0])
-     {
-     case 'a':
-     case 'A': 
-	  if (strcasecmp(field, "Architecture") == 0) {
-	       /* Architecture */
-	       if (pkg->architecture) {
-		    fprintf(file, "Architecture: %s\n", pkg->architecture);
-	       }
-	  }
-	  break;
-     case 'c':
-     case 'C': 
-	  if (strcasecmp(field, "Conffiles") == 0) {
-	       /* Conffiles */
-	       conffile_list_elt_t *iter;
-
-	       if (pkg->conffiles.head == NULL) {
-		    return;
-	       }
-
-	       fprintf(file, "Conffiles:\n");
-	       for (iter = pkg->conffiles.head; iter; iter = iter->next) {
-		    if (iter->data->name && iter->data->value) {
-			 fprintf(file, " %s %s\n", iter->data->name, iter->data->value);
-		    }
-	       }
-	  } else if (strcasecmp(field, "Conflicts") == 0) {
-	       int i;
-
-	       if (pkg->conflicts_count) {
-		    fprintf(file, "Conflicts:");
-		    for(i = 0; i < pkg->conflicts_count; i++)
-			 fprintf(file, "%s %s",
-				 i == 0 ? "" : ",",
-				 pkg->conflicts_str[i]);
-		    fprintf(file, "\n");
-	       }
-	       
-	  }
-	  break;
-     case 'd':
-     case 'D': {
-	  if (strcasecmp(field, "Depends") == 0) {
-	       /* Depends */
-	       int i;
-
-	       if (pkg->depends_count) {
-		    fprintf(file, "Depends:");
-		    for(i = 0; i < pkg->depends_count; i++)
-			 fprintf(file, "%s %s",
-				 i == 0 ? "" : ",",
-				 pkg->depends_str[i]);
-		    fprintf(file, "\n");
-	       }
-	  } else if (strcasecmp(field, "Description") == 0) {
-	       /* Description */
-	       if (pkg->description) {
-		    fprintf(file, "Description: %s\n", pkg->description);
-	       }
-	  } else {
-	       goto UNKNOWN_FIELD;
-	  }
-     } break;
-     case 'e':
-     case 'E': {
-	  /* Essential */
-	  if (pkg->essential) {
-	       fprintf(file, "Essential: yes\n");
-	  }
-     }
-	  break;
-     case 'f':
-     case 'F': {
-	  /* Filename */
-	  if (pkg->filename) {
-	       fprintf(file, "Filename: %s\n", pkg->filename);
-	  }
-     }
-	  break;
-     case 'i':
-     case 'I': {
-	  if (strcasecmp(field, "Installed-Size") == 0) {
-	       /* Installed-Size */
-	       fprintf(file, "Installed-Size: %s\n", pkg->installed_size);
-	  } else if (strcasecmp(field, "Installed-Time") == 0 && pkg->installed_time) {
-	       fprintf(file, "Installed-Time: %ul\n", pkg->installed_time);
-	  }
-     }
-	  break;
-     case 'm':
-     case 'M': {
-	  /* Maintainer | MD5sum */
-	  if (strcasecmp(field, "Maintainer") == 0) {
-	       /* Maintainer */
-	       if (pkg->maintainer) {
-		    fprintf(file, "Maintainer: %s\n", pkg->maintainer);
-	       }
-	  } else if (strcasecmp(field, "MD5sum") == 0) {
-	       /* MD5sum */
-	       if (pkg->md5sum) {
-		    fprintf(file, "MD5sum: %s\n", pkg->md5sum);
-	       }
-	  } else {
-	       goto UNKNOWN_FIELD;
-	  }
-     }
-	  break;
-     case 'p':
-     case 'P': {
-	  if (strcasecmp(field, "Package") == 0) {
-	       /* Package */
-	       fprintf(file, "Package: %s\n", pkg->name);
-	  } else if (strcasecmp(field, "Priority") == 0) {
-	       /* Priority */
-	       fprintf(file, "Priority: %s\n", pkg->priority);
-	  } else if (strcasecmp(field, "Provides") == 0) {
-	       /* Provides */
-	       int i;
-
-	       if (pkg->provides_count) {
-		    fprintf(file, "Provides:");
-		    for(i = 0; i < pkg->provides_count; i++)
-			 fprintf(file, "%s %s",
-				 i == 0 ? "" : ",",
-				 pkg->provides_str[i]);
-		    fprintf(file, "\n");
-	       }
-	  } else {
-	       goto UNKNOWN_FIELD;
-	  }
-     }
-	  break;
-
-     case 'r':
-     case 'R': {
-	  int i;
-	  /* Replaces | Recommends*/
-	  if (strcasecmp (field, "Replaces") == 0) {
-	       if (pkg->replaces_count) {
-		    fprintf(file, "Replaces:");
-		    for (i = 0; i < pkg->replaces_count; i++)
-			 fprintf(file, "%s %s",
-				 i == 0 ? "" : ",",
-				 pkg->replaces_str[i]);
-		    fprintf(file, "\n");
-	       }
-	  } else if (strcasecmp (field, "Recommends") == 0) {
-	       if (pkg->recommends_count) {
-		    fprintf(file, "Recommends:");
-		    for(i = 0; i < pkg->recommends_count; i++)
-			 fprintf(file, "%s %s",
-				 i == 0 ? "" : ",",
-				 pkg->recommends_str[i]);
-		    fprintf(file, "\n");
-	       }
-	  } else {
-	       goto UNKNOWN_FIELD;
-	  }
-     }
-	  break;
-
-     case 's':
-     case 'S': {
-	  /* Section | Size | Source | Status | Suggests */
-	  if (strcasecmp(field, "Section") == 0) {
-	       /* Section */
-	       if (pkg->section) {
-		    fprintf(file, "Section: %s\n", pkg->section);
-	       }
-	  } else if (strcasecmp(field, "Size") == 0) {
-	       /* Size */
-	       if (pkg->size) {
-		    fprintf(file, "Size: %s\n", pkg->size);
-	       }
-	  } else if (strcasecmp(field, "Source") == 0) {
-	       /* Source */
-	       if (pkg->source)
-		    fprintf(file, "Source: %s\n", pkg->source);
-	  } else if (strcasecmp(field, "Status") == 0) {
-	       /* Status */
-	       fprintf(file, "Status: %s %s %s\n",
-		       pkg_state_want_to_str(pkg->state_want),
-		       pkg_state_flag_to_str(pkg->state_flag),
-		       pkg_state_status_to_str(pkg->state_status));
-	  } else if (strcasecmp(field, "Suggests") == 0) {
-	       if (pkg->suggests_count) {
-		    int i;
-		    fprintf(file, "Suggests:");
-		    for(i = 0; i < pkg->suggests_count; i++)
-			 fprintf(file, "%s %s",
-				 i == 0 ? "" : ",",
-				 pkg->suggests_str[i]);
-		    fprintf(file, "\n");
-	       }
-	  } else {
-	       goto UNKNOWN_FIELD;
-	  }
-     }
-	  break;
-     case 'v':
-     case 'V': {
-	  /* Version */
-	  char *version = pkg_version_str_alloc(pkg);
-	  fprintf(file, "Version: %s\n", version);
-	  free(version);
-     }
-	  break;
-     default:
-	  goto UNKNOWN_FIELD;
-     }
-
-     return;
-
- UNKNOWN_FIELD:
-     fprintf(stderr, "%s: ERROR: Unknown field name: %s\n",
+       fprintf(stderr, "%s: ERROR: Unknown field name: %s\n",
 	     __FUNCTION__, field);
+     }
+     buff = pkg_formatted_field(pkg, field);
+     if (strlen(buff)>2) {
+       fprintf(file, "%s", buff);
+       fflush(file);
+     }
+     free(buff);
+     return;
 }
 
 /*
@@ -697,29 +1030,29 @@ int pkg_compare_versions(const pkg_t *pkg, const pkg_t *ref_pkg)
      if (pkg->epoch > ref_pkg->epoch) {
 	  return 1;
      }
-    
+
      if (pkg->epoch < ref_pkg->epoch) {
 	  return -1;
      }
-    
+
      r = verrevcmp(pkg->version, ref_pkg->version);
      if (r) {
 	  return r;
      }
-    
+
 #ifdef USE_DEBVERSION
      r = verrevcmp(pkg->revision, ref_pkg->revision);
      if (r) {
 	  return r;
      }
-    
+
      r = verrevcmp(pkg->familiar_revision, ref_pkg->familiar_revision);
 #endif
 
      return r;
 }
 
-static int verrevcmp(const char *val, const char *ref)
+int verrevcmp(const char *val, const char *ref)
 {
      int vc, rc;
      long vl, rl;
@@ -792,13 +1125,25 @@ int pkg_name_version_and_architecture_compare(void *p1, void *p2)
 {
      const pkg_t *a = *(const pkg_t **)p1;
      const pkg_t *b = *(const pkg_t **)p2;
-     int namecmp = strcmp(a->name, b->name);
+     int namecmp;
      int vercmp;
+     if (!a->name || !b->name) {
+       fprintf(stderr, "pkg_name_version_and_architecture_compare: a=%p a->name=%p b=%p b->name=%p\n",
+	       a, a->name, b, b->name);
+       return 0;
+     }
+       
+     namecmp = strcmp(a->name, b->name);
      if (namecmp)
 	  return namecmp;
      vercmp = pkg_compare_versions(a, b);
      if (vercmp)
 	  return vercmp;
+     if (!a->arch_priority || !b->arch_priority) {
+       fprintf(stderr, "pkg_name_version_and_architecture_compare: a=%p a->arch_priority=%p b=%p b->arch_priority=%p\n",
+	       a, a->arch_priority, b, b->arch_priority);
+       return 0;
+     }
      if (a->arch_priority > b->arch_priority)
 	  return 1;
      if (a->arch_priority < b->arch_priority)
@@ -810,6 +1155,11 @@ int abstract_pkg_name_compare(void *p1, void *p2)
 {
      const abstract_pkg_t *a = *(const abstract_pkg_t **)p1;
      const abstract_pkg_t *b = *(const abstract_pkg_t **)p2;
+     if (!a->name || !b->name) {
+       fprintf(stderr, "abstract_pkg_name_compare: a=%p a->name=%p b=%p b->name=%p\n",
+	       a, a->name, b, b->name);
+       return 0;
+     }
      return strcmp(a->name, b->name);
 }
 
@@ -867,6 +1217,7 @@ str_list_t *pkg_get_installed_files(pkg_t *pkg)
      FILE *list_file = NULL;
      char *line;
      char *installed_file_name;
+     int rootdirlen;
 
      pkg->installed_files_ref_cnt++;
 
@@ -918,6 +1269,7 @@ str_list_t *pkg_get_installed_files(pkg_t *pkg)
 	  free(list_file_name);
      }
 
+     rootdirlen = strlen( pkg->dest->root_dir );
      while (1) {
 	  char *file_name;
 	
@@ -929,15 +1281,22 @@ str_list_t *pkg_get_installed_files(pkg_t *pkg)
 	  file_name = line;
 
 	  /* Take pains to avoid uglies like "/./" in the middle of file_name. */
-	  if (*file_name == '.') {
-	       file_name++;
-	  }
-	  if (*file_name == '/') {
-	       file_name++;
-	  }
+	  if( strncmp( pkg->dest->root_dir, 
+		       file_name, 
+		       rootdirlen ) ) {
+	       if (*file_name == '.') {
+		    file_name++;
+	       }
+	       if (*file_name == '/') {
+		    file_name++;
+	       }
 
-	  /* Freed in pkg_free_installed_files */
-	  sprintf_alloc(&installed_file_name, "%s%s", pkg->dest->root_dir, file_name);
+	       /* Freed in pkg_free_installed_files */
+	       sprintf_alloc(&installed_file_name, "%s%s", pkg->dest->root_dir, file_name);
+	  } else {
+	       // already contains root_dir as header -> ABSOLUTE
+	       sprintf_alloc(&installed_file_name, "%s", file_name);
+	  }
 	  str_list_append(pkg->installed_files, installed_file_name);
 	  free(line);
      }
@@ -1134,6 +1493,10 @@ char *pkg_state_flag_to_str(pkg_state_flag_t sf)
 	       }
 	  }
 	  str = malloc(len);
+          if ( str == NULL ) {
+	      fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+              return NULL;
+          }
 	  str[0] = 0;
 	  for (i=0; i < ARRAY_SIZE(pkg_state_flag_map); i++) {
 	       if (sf & pkg_state_flag_map[i].value) {
@@ -1320,12 +1683,25 @@ int pkg_write_filelist(ipkg_conf_t *conf, pkg_t *pkg)
      char *list_file_name = NULL;
      int err = 0;
 
+     if (!pkg) {
+	  ipkg_message(conf, IPKG_ERROR, "Null pkg\n");
+	  return -EINVAL;
+     }
      ipkg_message(conf, IPKG_NOTICE,
 		  "    creating %s.list file\n", pkg->name);
      sprintf_alloc(&list_file_name, "%s/%s.list", pkg->dest->info_dir, pkg->name);
+     if (!list_file_name) {
+	  ipkg_message(conf, IPKG_ERROR, "Failed to alloc list_file_name\n");
+	  return -ENOMEM;
+     }
      ipkg_message(conf, IPKG_NOTICE,
 		  "    creating %s file for pkg %s\n", list_file_name, pkg->name);
      data.stream = fopen(list_file_name, "w");
+     if (!data.stream) {
+	  ipkg_message(conf, IPKG_ERROR, "Could not open %s for writing: %s\n",
+		       list_file_name, strerror(errno));
+		       return errno;
+     }
      data.pkg = pkg;
      data.conf = conf;
      hash_table_foreach(&conf->file_hash, pkg_write_filelist_helper, &data);
@@ -1349,6 +1725,7 @@ int pkg_write_changed_filelists(ipkg_conf_t *conf)
      for (i = 0; i < installed_pkgs->len; i++) {
 	  pkg_t *pkg = installed_pkgs->pkgs[i];
 	  if (pkg->state_flag & SF_FILELIST_CHANGED) {
+               ipkg_message(conf, IPKG_DEBUG, "Calling pkg_write_filelist for pkg=%s from %s\n", pkg->name, __FUNCTION__);
 	       err = pkg_write_filelist(conf, pkg);
 	       if (err)
 		    ipkg_message(conf, IPKG_NOTICE, "pkg_write_filelist pkg=%s returned %d\n", pkg->name, err);
